# Generated by roxygen2: do not edit by hand

S3method("[",listSubclass)
S3method(as.data.frame,listOfBlueprints)
S3method(as.data.frame,listOfDataRobotPredictionDatasets)
S3method(as.data.frame,listOfFeaturelists)
S3method(as.data.frame,listOfModels)
S3method(as.data.frame,projectSummaryList)
S3method(plot,listOfModels)
S3method(predict,dataRobotModel)
S3method(summary,dataRobotModel)
S3method(summary,dataRobotProject)
S3method(summary,listOfBlueprints)
S3method(summary,listOfDataRobotTuningParameters)
S3method(summary,listOfFeaturelists)
S3method(summary,listOfModels)
S3method(summary,projectSummaryList)
export(AddEureqaSolution)
export(AutopilotMode)
export(BatchFeaturesTypeTransform)
export(BlendMethods)
export(BlueprintChartToGraphviz)
export(ClassificationDeploymentAccuracyMetric)
export(CloneProject)
export(ConnectToDataRobot)
export(ConstructDurationString)
export(CreateBacktestSpecification)
export(CreateCalendar)
export(CreateComplianceDocumentation)
export(CreateDataSource)
export(CreateDataStore)
export(CreateDatetimePartitionSpecification)
export(CreateDeployment)
export(CreateDerivedFeatureAsCategorical)
export(CreateDerivedFeatureAsNumeric)
export(CreateDerivedFeatureAsText)
export(CreateDerivedFeatureIntAsCategorical)
export(CreateFeaturelist)
export(CreateGroupPartition)
export(CreateModelingFeaturelist)
export(CreatePrimeCode)
export(CreateRandomPartition)
export(CreateRatingTable)
export(CreateStratifiedPartition)
export(CreateUserPartition)
export(CrossValidateModel)
export(DataPartition)
export(DataSubset)
export(DeleteAnomalyAssessmentRecord)
export(DeleteCalendar)
export(DeleteComplianceDocTemplate)
export(DeleteDataSource)
export(DeleteDataStore)
export(DeleteDeployment)
export(DeleteFeaturelist)
export(DeleteJob)
export(DeleteModel)
export(DeleteModelJob)
export(DeleteModelingFeaturelist)
export(DeletePredictJob)
export(DeletePredictionDataset)
export(DeletePredictionExplanations)
export(DeletePredictionExplanationsInitialization)
export(DeleteProject)
export(DeleteTransferableModel)
export(DeploymentAccuracyMetric)
export(DeploymentServiceHealthMetric)
export(DifferencingMethod)
export(DownloadComplianceDocTemplate)
export(DownloadComplianceDocumentation)
export(DownloadPredictionExplanations)
export(DownloadPrimeCode)
export(DownloadRatingTable)
export(DownloadScoringCode)
export(DownloadSeriesAccuracy)
export(DownloadTimeSeriesFeatureDerivationLog)
export(DownloadTrainingPredictions)
export(DownloadTransferableModel)
export(FeatureFromAsyncUrl)
export(GenerateDatetimePartition)
export(GetAnomalyAssessmentExplanations)
export(GetAnomalyAssessmentPredictionsPreview)
export(GetBlenderModel)
export(GetBlenderModelFromJobId)
export(GetBlueprint)
export(GetBlueprintChart)
export(GetBlueprintDocumentation)
export(GetCalendar)
export(GetCalendarFromProject)
export(GetComplianceDocTemplate)
export(GetConfusionChart)
export(GetCrossValidationScores)
export(GetDataSource)
export(GetDataStore)
export(GetDataStoreSchemas)
export(GetDataStoreTables)
export(GetDatetimeModel)
export(GetDatetimeModelFromJobId)
export(GetDatetimePartition)
export(GetDeployment)
export(GetDeploymentAccuracy)
export(GetDeploymentAccuracyOverTime)
export(GetDeploymentAssociationId)
export(GetDeploymentDriftTrackingSettings)
export(GetDeploymentServiceStats)
export(GetDeploymentServiceStatsOverTime)
export(GetDeploymentSettings)
export(GetDriver)
export(GetFeatureAssociationMatrix)
export(GetFeatureAssociationMatrixDetails)
export(GetFeatureHistogram)
export(GetFeatureImpact)
export(GetFeatureImpactForJobId)
export(GetFeatureImpactForModel)
export(GetFeatureInfo)
export(GetFeaturelist)
export(GetFrozenModel)
export(GetFrozenModelFromJobId)
export(GetJob)
export(GetLiftChart)
export(GetMissingValuesReport)
export(GetModel)
export(GetModelBlueprintChart)
export(GetModelBlueprintDocumentation)
export(GetModelCapabilities)
export(GetModelFromJobId)
export(GetModelJob)
export(GetModelParameters)
export(GetModelRecommendation)
export(GetModelingFeaturelist)
export(GetMultiSeriesProperties)
export(GetParetoFront)
export(GetPredictJob)
export(GetPredictJobs)
export(GetPredictionDataset)
export(GetPredictionExplanations)
export(GetPredictionExplanationsInitialization)
export(GetPredictionExplanationsInitializationFromJobId)
export(GetPredictionExplanationsMetadata)
export(GetPredictionExplanationsMetadataFromJobId)
export(GetPredictionExplanationsRows)
export(GetPredictionExplanationsRowsAsDataFrame)
export(GetPredictions)
export(GetPrimeEligibility)
export(GetPrimeFile)
export(GetPrimeFileFromJobId)
export(GetPrimeModel)
export(GetPrimeModelFromJobId)
export(GetProject)
export(GetProjectStatus)
export(GetRatingTable)
export(GetRatingTableFromJobId)
export(GetRatingTableModel)
export(GetRatingTableModelFromJobId)
export(GetRecommendedModel)
export(GetResidualsChart)
export(GetRocCurve)
export(GetRulesets)
export(GetSeriesAccuracy)
export(GetSeriesAccuracyForModel)
export(GetTimeSeriesFeatureDerivationLog)
export(GetTrainingPredictions)
export(GetTrainingPredictionsForModel)
export(GetTrainingPredictionsFromJobId)
export(GetTransferableModel)
export(GetTuningParameters)
export(GetValidMetrics)
export(GetWordCloud)
export(InitializeAnomalyAssessment)
export(IsBlenderEligible)
export(JobStatus)
export(JobType)
export(ListAnomalyAssessmentRecords)
export(ListBlueprints)
export(ListCalendars)
export(ListComplianceDocTemplates)
export(ListConfusionCharts)
export(ListDataSources)
export(ListDataStores)
export(ListDeployments)
export(ListDrivers)
export(ListFeatureInfo)
export(ListFeaturelists)
export(ListJobs)
export(ListLiftCharts)
export(ListModelFeatures)
export(ListModelJobs)
export(ListModelRecommendations)
export(ListModelingFeaturelists)
export(ListModels)
export(ListPredictionDatasets)
export(ListPredictionExplanationsMetadata)
export(ListPredictionServers)
export(ListPredictions)
export(ListPrimeFiles)
export(ListPrimeModels)
export(ListProjects)
export(ListRatingTableModels)
export(ListRatingTables)
export(ListResidualsCharts)
export(ListRocCurves)
export(ListSharingAccess)
export(ListStarredModels)
export(ListTrainingPredictions)
export(ListTransferableModels)
export(ModelCapability)
export(ModelReplacementReason)
export(MulticlassDeploymentAccuracyMetric)
export(PauseQueue)
export(PeriodicityMaxTimeStep)
export(PeriodicityTimeUnits)
export(PostgreSQLdrivers)
export(Predict)
export(PredictionDatasetFromAsyncUrl)
export(PrimeLanguage)
export(ProjectFromJobResponse)
export(ProjectStage)
export(RecommendedModelType)
export(RegressionDeploymentAccuracyMetric)
export(RenameRatingTable)
export(ReplaceDeployedModel)
export(RequestApproximation)
export(RequestBlender)
export(RequestCrossSeriesDetection)
export(RequestFeatureImpact)
export(RequestFrozenDatetimeModel)
export(RequestFrozenModel)
export(RequestMultiSeriesDetection)
export(RequestNewDatetimeModel)
export(RequestNewModel)
export(RequestNewRatingTableModel)
export(RequestPredictionExplanations)
export(RequestPredictionExplanationsInitialization)
export(RequestPredictions)
export(RequestPrimeModel)
export(RequestSampleSizeUpdate)
export(RequestSeriesAccuracy)
export(RequestTrainingPredictions)
export(RequestTransferableModel)
export(RunInteractiveTuning)
export(ScoreBacktests)
export(SegmentAnalysisAttribute)
export(SeriesAggregationType)
export(SetPredictionThreshold)
export(SetTarget)
export(SetupProject)
export(SetupProjectFromDataSource)
export(SetupProjectFromHDFS)
export(Share)
export(SharingRole)
export(SourceType)
export(StarModel)
export(StartNewAutoPilot)
export(StartProject)
export(StartTuningSession)
export(SubmitActuals)
export(TargetLeakageType)
export(TargetType)
export(TestDataStore)
export(TimeUnits)
export(ToggleStarForModel)
export(TreatAsExponential)
export(UnpauseQueue)
export(UnstarModel)
export(UpdateAccess)
export(UpdateCalendar)
export(UpdateComplianceDocTemplate)
export(UpdateDataSource)
export(UpdateDataStore)
export(UpdateDeploymentAssociationId)
export(UpdateDeploymentDriftTrackingSettings)
export(UpdateFeaturelist)
export(UpdateModelingFeaturelist)
export(UpdateProject)
export(UpdateTransferableModel)
export(UploadComplianceDocTemplate)
export(UploadPredictionDataset)
export(UploadPredictionDatasetFromDataSource)
export(UploadTransferableModel)
export(ValidateMultiSeriesProperties)
export(ValidateReplaceDeployedModel)
export(VariableTransformTypes)
export(ViewWebModel)
export(ViewWebProject)
export(WaitForAutopilot)
export(WaitForJobToComplete)
export(cvMethods)
import(graphics)
import(methods)
import(utils)
